# This module is part of the GAP package BRAID.
# It contains a program computing the index of 
# permutations and the genus of a generating 
# set of a group.
#
# (C) K. Magaard, S. Shpectorov and H. Voelklein, 2001

PermIndex:=function(p,deg)

 return deg - Length(Orbits(Group(p),[1..deg]));

end;

OrbitGenera:=function(t,deg)
 local p,g;

 p:=Concatenation(t,[Product(t)^-1]);
 g:=Group(p);
 return List(Orbits(g,[1..deg]),x->[Length(x),1-Length(x)+
         Sum(List([1..Length(p)],
          i->PermIndex(GeneratorsOfGroup(Action(g,x))[i],Length(x))))/2]);

end;              

ReducedGenus:=function(P,b, deg)
 local s,v,d,i1,i2,i3,qq,oo,bb1,bb13,bb3; 


 if P = [1,1,1,1] then
 s := [b[1][4],b[2][4],b[3][4]];
 if Product(s) <> () then 
  return Print("Can't compute ReducedGenus since product is not ()");
 else
   return OrbitGenera(s,deg)[1][2];
 fi;
 fi;

if P = [3,1] then
 s := [b[1][1]*b[2][2],b[1][1]*b[2][2]*b[1][1],b[2][2]];
  if Product(s) <> () then
    return Print("Can't compute ReducedGenus since product is not ()");
     else
        if IsTransitive(Group(s),[1..deg]) then
	   return OrbitGenera(s,deg)[1][2];
	     else
    return Print("Can't compute ReducedGenus since Group(s) is not transitive");
   fi;
   fi;
fi;

if P = [4] then
  s := [b[1][1]*b[2][2],b[1][1]*b[2][2]*b[1][1],b[2][2]];  
  v:= Group((b[1][1]*b[2][2]*b[3][3])^2,b[1][1]*b[3][3]^-1);
  if Size(v) in [1,4] then
  s:=Action(Group(s),List(Orbits(v,[1..deg]),Set),OnSets);
  s:=GeneratorsOfGroup(s);
  if Product(s) <> () then
   return Print("Can't compute ReducedGenus since product is not ()");
   else
    d := LargestMovedPoint(Group(s));
  if IsTransitive(Group(s),[1..d]) then
   return OrbitGenera(s,d)[1][2];
  else
    return Print("Can't compute ReducedGenus since Group(s) is not transitive");
  fi;
 fi;
  else
  return Print("Can't compute ReducedGenus since Order(v) is not 1 or 4");
 fi;
fi;
 
 if P = [2,1,1] then 
 i1:=PermIndex(b[1][1],deg) + PermIndex(b[1][3],deg)+ deg; 
 i2:=2*deg;
 i3:= PermIndex(b[2][3]^-1*b[1][1],deg) + deg;
# i3 is the index of the element q2*(q1*q2*q1)*q2^-1 = q2^2*q1 on reduced 
# (unordered) Nielsen classes.
return (i1+i2+i3 - 6*deg + 2)/2;
fi;

if P = [2,2] then 
  d:= PermIndex(b[1][1]*b[3][3]^-1,deg);
  if not (d in [0,deg/2]) then 
  return Print("Can't compute ReducedGenus as quaternion groups acts improperly");
  fi;
	   
  if d= 0 then 
#    Print("d = 0");
    i1:=PermIndex(b[1][1],deg) + PermIndex(b[1][3],deg)+ deg;
    i2:=2*deg;
    i3:= PermIndex(b[2][3]^-1*b[1][1],deg) + deg;
# i3 is the index of the involution q2*(q1*q2*q1)*q2^-1 = q2^2*q1 on reduced 
# (unordered) Nielsen classes. The inverse is becasue we compute 
# q_i^-1 rather than q_i. 
    return (i1+i2+i3 - 6*deg + 2)/2;
   fi;
					      
					        
  if d = deg/2 then 
   qq:=b[1][1]*b[3][3]^-1;
   oo:= Orbits(Group(qq),[1..deg]);
   bb1:=Action(Group(b[1][1]),oo,OnSets).1;
   bb13:=Action(Group(b[1][3]),oo,OnSets).1;
   bb3:=Action(Group(b[2][3]^-1*b[1][1]),oo,OnSets).1;
   i1:=PermIndex(bb1,deg/2) +PermIndex(bb13,deg/2) + deg/2;
   i2:=deg;
   i3:= PermIndex(bb3,deg/2) + deg/2;
   return (i1+i2+i3 - 3*deg + 2)/2;
  fi;
fi;

end; 


Braid:=function(arg)

local G, GG, N, U, act, t, pp, str, chartab, rep, tt, s, d, jj, cc, gg, isinner; 

Exec("rm -rf _TEMP");

if Length(arg) = 2 then 
  G:=arg[1];
  t:=arg[2];
  isinner:=true;
  N:=G;
else 
  GG:=arg[1];
  tt:=arg[2];
  U:=arg[3];
  G:=Action(GG,RightCosets(GG,U),OnRight);
  act:=ActionHomomorphism(GG,RightCosets(GG,U),OnRight);
  t:=List(tt,x->Image(act,x)); 
  N:=Normalizer(SymmetricGroup(Index(GG,U)),G);
  isinner:=false;
fi;

#Order tuple and compute partition

 tt:=[];pp:=[];
 s:=ShallowCopy(t);
   while Length(s) <> 0 do
   jj:=s[1];
   d:=Length(s); 
   s:=Filtered(s,x-> not IsConjugate(N,x,jj));
   d:= d-Length(s);
   Append(pp,[d]);
   Append(tt,[List([1..d],x->jj)]);
   od;
   SortParallel(pp,tt);
   pp:=Reversed(pp);
  tt:=Reversed(tt);
   s:=ShallowCopy(tt);
  tt:=[];
  for jj in [1..Length(s)] do 
   Append(tt,s[jj]);
  od;
						       

 #character table and structure constant
  chartab:=CharacterTable(N);
  rep:=List(ConjugacyClasses(chartab),x->Representative(x));
   cc:=List(tt,x->Position(List(rep,y->IsConjugate(N,x,y)),true));
  str:=ClassStructureCharTable(chartab,cc);

 #computing the braid orbits

  BraidOrbits("TEMP",N,tt,pp,str);
					    
 #results summary
Print("Summary: orbits of generating tuples","\n");
   cc:=0;
    for jj in [1..NumberOfOrbits] do 
      RestoreOrbit(jj);
     if Size(G) = Size(Group(TupleTable[1].tuple)) then
      cc:=cc+1; 
      Print("\n");
      Print("Orbit ", cc, " of Length ", Length(TupleTable),"\n");
       if Length(tt) = 4 then 
         ExtractPermutations(); 
#         RetrievePureBraidPermutations();
         gg:=ReducedGenus(pp,BraidPermutation,Length(TupleTable));
        if isinner then
          Print("Inner Hurwitz curve genus = ", gg,"\n");
          gg:=ReducedGenus([1,1,1,1],BraidPermutation,Length(TupleTable));
          Print("Straight inner Hurwitz curve genus = ", gg,"\n \n");
        else
          Print("Absolute Hurwitz curve genus = ", gg, "\n\n");
        fi;
       fi;
      fi;
    od;
										      
end;

BraidGenus:=function(G,t,U)

local NN, GG, act, tt; 

GG:=Action(G,RightCosets(G,U),OnRight);
act:=ActionHomomorphism(G,RightCosets(G,U),OnRight);
tt:=Image(act,t);
NN:=Normalizer(SymmetricGroup(Index(G,U)),GG);
Braid(NN,tt,GG);

end; 

# End
