# This module is part of a GAP package BRAID. 
# It deals with maintaining a list of small subgroups. 
# 
# (c) K. Magaard, S. Shpectorov, 2001 
 
 
# Putting a new subgroup on the list
 
CheckInSubgroup:=function(s) 
 local r,l,i;
 
 r:=List(ConjugacyClasses(s),x->Representative(x)); 

 l:=[]; 
 for i in [2..Length(PrincipalTuple)] do 
  Append(l,[Filtered(r,x->IsConjugate(PrincipalFiniteGroup,x,PrincipalTuple[i]))]);
 od;

 Append(SmallSubgroups,[rec(subgroup:=s,classes:=l,weight:=InitialWeight)]);
 TotalWeight:=TotalWeight+InitialWeight;

end; 


# Check whether or not a subgroup is on the list

IdentifySubgroup:=function(s)
 local r;

 if Size(s)>SizeCutoff then
  TotalWeight:=TotalWeight-SmallSubgroups[1].weight;
  SmallSubgroups[1].weight:=MinimumList([SmallSubgroups[1].weight+BumpUp,MaximumWeight]);
  TotalWeight:=TotalWeight+SmallSubgroups[1].weight;
  return;
 fi;
 for r in SmallSubgroups{[2..Length(SmallSubgroups)]} do
  if Size(r.subgroup)=Size(s) then
   if RepresentativeAction(Centralizer(PrincipalFiniteGroup,s.1),r.subgroup,s)<>fail then
    TotalWeight:=TotalWeight-r.weight;
    r.weight:=MinimumList([r.weight+BumpUp,MaximumWeight]);
    TotalWeight:=TotalWeight+r.weight;
    return;
   fi;
  fi;
 od;
 CheckInSubgroup(s);
 
end;
 
 
#  Select a random tuple in a random subgroup

RandomSubgroupTuple:=function()
 local r,i,s,T,p,c,x;
 
 r:=RandomList([1..TotalWeight]);
 for i in [1..Length(SmallSubgroups)] do
  s:=SmallSubgroups[i];
  if s.weight<r then
   r:=r-s.weight;
  else   
   while true do 
    T:=[PrincipalTuple[1]];
    for c in s.classes{[1..Length(s.classes)-1]} do
     Append(T,[RandomList(c)^Random(s.subgroup)]);
    od;
    p:=Product(T)^-1;
    c:=s.classes[Length(s.classes)];
    for x in c do 
     if IsConjugate(s.subgroup,p,x) then
      Append(T,[p]);
      return rec(tuple:=T,subgroupNumber:=i);
     fi;
    od;
   od;
  fi;
 od;
 
end;


# End 

