# This module is part of a GAP package BRAID.
# It contains the main function computing all
# pure braid orbits of a given type. 
#
# (C) K. Magaard, S. Shpectorov, 2001

#
# function inserting a human-suggested tuple
#

SuggestTuple:=function(tu)

 local i;

 for i in [1..Length(PrincipalTuple)] do
  if not IsConjugate(PrincipalFiniteGroup,tu[i],PrincipalTuple[i]) then
   Print("\n\nTuple cannot be accepted!!\n\n");
   return;
  fi;
 od;
 if Product(tu)<>One(PrincipalFiniteGroup) then
  Print("\n\nTuple cannot be accepted!!\n\n");
  return;
 fi;
 Append(SuggestedTuples,[rec(tuple:=ShallowCopy(tu),subgroupNumber:=1)]);

end;


#
# Function collecting random tuples
#

CollectRandomTuples:=function()
 Print("\n\nCollecting ",NumberOfRandomTuples," random tuples...\c");
 while Length(RandomTuples)<NumberOfRandomTuples do
  Append(RandomTuples,[RandomSubgroupTuple()]);
 od;
 Print(" done\n");
end;


#
# Two routines cleaning the list of random tuples
#

CleanCurrent:=function()
 local i,l,s; 
 for i in [1..Length(RandomTuples)] do
  l:=Length(TupleTable);
  LookUpTuple(RandomTuples[i].tuple);
  if l=Length(TupleTable) then
   s:=SmallSubgroups[RandomTuples[i].subgroupNumber];
   TotalWeight:=TotalWeight-s.weight;
   s.weight:=MaximumList([MinimumWeight_braid,s.weight-KnockDown]);
   TotalWeight:=TotalWeight+s.weight;
   Unbind(RandomTuples[i]);
  fi;
 od;
 RandomTuples:=Filtered(RandomTuples,x->IsBound(x));
end;


CleanAll:=function()
 local i;
 for i in [1..NumberOfOrbits] do
  Print("\rCleaning orbit ",i,"\c");
  RestoreOrbit(i);
  CleanCurrent();
 od;
 Print("\r Cleaning done; ",Length(RandomTuples)," random tuples remaining\n");
end;


#
# Main routine
#

BraidOrbits:=function(project,GG,CC,PP,stc)
 local i,t,g,c,r;

 Exec("date");
 Print("\n");

#
# Initialize 
#

# project name...
 ProjectName:=project;
 Exec(Concatenation("mkdir _",ProjectName));

# group...
 PrincipalFiniteGroup:=GG;

# tuple...
 PrincipalTuple:=CC;

# number of generators...
 NumberOfGenerators_braid:=Length(PrincipalTuple); # mis-spelled on purpose to avoid name pollution

# abstract generators...
 OurFreeGroup:=FreeGroup(NumberOfGenerators_braid);
 AbsGens:=GeneratorsOfGroup(OurFreeGroup);
 for i in [1..NumberOfGenerators_braid] do
  AppendTo("_tmptmp_","f",String(i),":=AbsGens[",String(i),"];\n");
 od;
 Read("_tmptmp_");
 Exec("rm _tmptmp_");

# braid and pure braid group generators...
 BraidGroupGenerators();

# tuple partition...
 TuplePartition:=[PP[1]];
 for i in [2..Length(PP)] do
  TuplePartition[i]:=TuplePartition[i-1]+PP[i];
 od;

# number of orbits...
 NumberOfOrbits:=0;
 
# initialize SmallSubgroups, TotalWeight and SizeCutoff...
 SmallSubgroups:=[rec(subgroup:=PrincipalFiniteGroup,
        classes:=List(PrincipalTuple{[2..Length(PrincipalTuple)]},x->[x]),
        weight:=InitialWeight)];
 TotalWeight:=InitialWeight;
 SizeCutoff:=Size(PrincipalFiniteGroup)-1;
 
# list of random tuples...
 RandomTuples:=[];

# number of tuples remaining...
 t:=stc;

# number of orbits before last collection
 r:=0;

#
# main loop
#

 while t>CutOffValue do
  if Length(RandomTuples)=0 then
   if Length(SuggestedTuples)=0 then
    if NumberOfOrbits=r and r<>0 then
     NumberOfRandomTuples:=NumberOfRandomTuples*2;
    fi;
    r:=NumberOfOrbits;
    CollectRandomTuples();
   else
    RandomTuples:=ShallowCopy(SuggestedTuples);
   fi;
   CleanAll();
  else
   Print("\n\nOrbit ",NumberOfOrbits+1,":\n");
   BraidOrbit(RandomTuples[1].tuple);
   Print("Length=",Length(TupleTable),"\n");
   SaveOrbit();
   g:=RandomTuples[1].subgroupNumber;
   TotalWeight:=TotalWeight-SmallSubgroups[g].weight;
   SmallSubgroups[g].weight:=MinimumList([MaximumWeight,SmallSubgroups[g].weight+BumpUp]);
   TotalWeight:=TotalWeight+SmallSubgroups[g].weight;
   g:=Subgroup(PrincipalFiniteGroup,RandomTuples[1].tuple);
   Print("Generated subgroup size=",Size(g),"\n");
   IdentifySubgroup(g);
   c:=Centralizer(PrincipalFiniteGroup,g);
   Print("Centralizer size=",Size(c),"\n");
   t:=t-Length(TupleTable)*Index(PrincipalFiniteGroup,c);
   Print(t," tuples remaining\n");
   Print("Cleaning current orbit...\c");
   CleanCurrent();
   Print(" done; ",Length(RandomTuples)," random tuples remaining\n");
  fi;
 od;
 
 Print("\n");
 Exec("date");

 NumberOfRandomTuples:=InitialNumberOfRandomTuples;
 CutOffValue:=InitialCutOffValue;

end;


# End


