# This module is part of a GAP package BRAID. 
# It contains a function computing the pure 
# braid orbit of a tuple and the action of 
# the pure braid group on the orbit
#
# (C) K. Magaard, S. Shpectorov, 2001


# ASSUMPTIONS: (1) The tuple multiplies to 1.
#              (2) The length of the tuple is >1.


BraidOrbit:= function(TT)
 local k,i,j,s,T,r;

#
# Initializing the work space
#

# list of test words...
 TestWord:= [];
 for i in [1..NumberOfTestWords] do
  Append(TestWord,[RandomWord()]);
 od;

# tables...
 InitializeTable();
 
# first tuple...
 LookUpTuple(TT);

# braid action...
 for i in [1..NumberOfGenerators_braid-1] do
  BraidAction[i]:= [];
  for j in [i..NumberOfGenerators_braid] do
   if (i=j and not (i in TuplePartition)) or
      (i<j and (i in TuplePartition) and (j-1 in TuplePartition))
   then 
    BraidAction[i][j]:=[];
   fi;
  od;
 od;
  

# 
# Generating the orbit
#

 k:=0;
 while k<Length(TupleTable) do
  k:=k+1;
  for i in [1..NumberOfGenerators_braid-1] do
   for j in [i..NumberOfGenerators_braid] do
    if IsBound(BraidAction[i][j]) then
     T:=[];
     for s in [1..NumberOfGenerators_braid] do
      T[s]:=MappedWord(PQ[i][j][s],AbsGens,TupleTable[k].tuple);
     od;
     if T[1]<>TupleTable[k].tuple[1] then
      r:=RepresentativeAction(PrincipalFiniteGroup,T[1],TupleTable[k].tuple[1]);
      T:=List(T,x->x^r);
     fi;
     BraidAction[i][j][k]:=LookUpTuple(T);
    fi;
   od;
  od;
  Print("\r",k," points checked, ",Length(TupleTable),
        " points total (difference ",Length(TupleTable)-k,")\c");
 od;
 Print("\r",List([1..70],x->' '),"\c\r");

end;


# End

