# This module is part of a GAP package BRAID.
# It contains a program extracting permutations
# corresponding to the pure braid group
# generators.
#
# (C) K. Magaard, S. Shpectorov, 2001


# Function recovering the action of the braid generators
# on the orbit

ExtractPermutations:=function()
 local i,j;
 BraidPermutation:=[];
 for i in [1..NumberOfGenerators_braid-1] do
  BraidPermutation[i]:=[];
  for j in [i..NumberOfGenerators_braid] do
   if IsBound(BraidAction[i][j]) then
    BraidPermutation[i][j]:=PermList(BraidAction[i][j]);
   fi;
  od;
 od;
end; 

# Function recovering the images of the generators of the 
# pure braid group. 

RetrievePureBraidPermutations:=function()
 local i,j;
 for i in [NumberOfGenerators_braid-1,NumberOfGenerators_braid-2..1] do
  for j in [i+1..NumberOfGenerators_braid] do
   if not IsBound(BraidPermutation[i][j]) then
    if j-i=1 then  
     BraidPermutation[i][j]:=BraidPermutation[i][i]^2;
    else
     if IsBound(BraidPermutation[j-1][j-1]) then
      BraidPermutation[i][j]:=BraidPermutation[j-1][j-1]*
            BraidPermutation[i][j-1]*BraidPermutation[j-1][j-1]^-1;
     else
      BraidPermutation[i][j]:=BraidPermutation[i][i]^-1*
            BraidPermutation[i+1][j]*BraidPermutation[i][i];   
     fi; 
    fi;
   fi;
  od;
 od;

end;  

# End

