# This module is part of a GAP package BRAID.
# It contains the function constructing 
# the generators of the pure braid group.
#
# (C) K. Magaard, S. Shpectorov, 2001


# Function producing the generators of the pure braid group

BraidGroupGenerators:=function()
 local i,j,k,s,Q,Qinv,PQcopy;

# getting the generators of the braid group 
 Q:=[];
 for i in [1..NumberOfGenerators_braid-1] do
  Q[i]:=[];
  for j in [1..i-1] do
   Q[i][j]:=AbsGens[j];
  od;
  Q[i][i]:=AbsGens[i]*AbsGens[i+1]*AbsGens[i]^-1;
  Q[i][i+1]:=AbsGens[i];
  for j in [i+2..NumberOfGenerators_braid] do
   Q[i][j]:=AbsGens[j];
  od;
 od;

# and their inverses
 Qinv:=[];
 for i in [1..NumberOfGenerators_braid-1] do
  Qinv[i]:=[];
  for j in [1..i-1] do
   Qinv[i][j]:=AbsGens[j];
  od;
  Qinv[i][i]:=AbsGens[i+1];
  Qinv[i][i+1]:=AbsGens[i+1]^-1*AbsGens[i]*AbsGens[i+1];
  for j in [i+2..NumberOfGenerators_braid] do
   Qinv[i][j]:=StructuralCopy(AbsGens[j]);
  od;
 od;

# finally, the pure braid group generators
 PQ:=[];
 for i in [1..NumberOfGenerators_braid-1] do
  PQ[i]:=[];
  PQ[i][i]:=ShallowCopy(Q[i]);
  for j in [i+1..NumberOfGenerators_braid] do
   PQ[i][j]:=List(AbsGens);
   for k in [i..j-2] do
    PQcopy:=ShallowCopy(PQ[i][j]);
    for s in [1..NumberOfGenerators_braid] do
     PQ[i][j][s]:=MappedWord(Q[k][s],AbsGens,PQcopy);
    od;
   od;
   PQcopy:=ShallowCopy(PQ[i][j]);
   for s in [1..NumberOfGenerators_braid] do
    PQ[i][j][s]:=MappedWord(Qinv[j-1][s],AbsGens,PQcopy);
   od;
   PQcopy:=ShallowCopy(PQ[i][j]);
   for s in [1..NumberOfGenerators_braid] do
    PQ[i][j][s]:=MappedWord(Qinv[j-1][s],AbsGens,PQcopy);
   od;
   for k in [i..j-2] do
    PQcopy:=ShallowCopy(PQ[i][j]);
    for s in [1..NumberOfGenerators_braid] do
     PQ[i][j][s]:=MappedWord(Qinv[j-2+i-k][s],AbsGens,PQcopy);
    od;
   od;
  od;
 od;

# we normalize the off-diagonal generators so that they don't change 
# the first entry
 for i in [2..NumberOfGenerators_braid] do
  for j in [1..NumberOfGenerators_braid] do
   PQ[1][i][j]:=AbsGens[i]*PQ[1][i][j]*AbsGens[i]^-1;
  od;
 od;
end;


# End

