# This module is part of a GAP package BRAID. 
# It contains functions maintaining a hash table 
# containing a braid orbit.
#
# (C) K. Magaard, S. Shpectorov, 2001


# Function initializing tables 

InitializeTable:=function()
 TupleTable:=[];
 FingerprintTable:=[];
 Hash:=List([1..HashLength],x->0);
 TestWord:=List([1..NumberOfTestWords],x->RandomWord());
 PrimeCode:=List([1..NumberOfTestWords],x->RandomList(Primes));
end;


# Function finding the fingerprint of a tuple

FingerprintOfTuple:=function(T)
 return List(TestWord,x->Order(MappedWord(x,AbsGens,T)));
end;


# Function computing the hash key of a fingerprint

HashKey:=function(F)
 return 
  1+(Sum(List([1..NumberOfTestWords],x->F[x]*PrimeCode[x])) mod HashLength);
end;


# Function looking up a fingerprint in the table (and adding it if not found)

LookUpFingerprint:= function(F)
 local h,a;
 h:=HashKey(F);
 a:=Hash[h];
 while a<>0 do
  if FingerprintTable[a].fingerprint=F then
   return a;
  else
   a:=FingerprintTable[a].next;
  fi;
 od;
 Append(FingerprintTable,[rec(fingerprint:=F,next:=Hash[h],firstTuple:=0)]);
 Hash[h]:=Length(FingerprintTable);
 return Hash[h];
end;


# Function looking up a tuple (and adding it to the table if not found)

LookUpTuple:=function(T)
 local f,a;
 f:=LookUpFingerprint(FingerprintOfTuple(T));
 a:=FingerprintTable[f].firstTuple;
 while a<>0 do
  if RepresentativeAction(PrincipalFiniteGroup,
                          TupleTable[a].tuple,T,OnTuples)<>fail then
   return a;
  else 
   a:=TupleTable[a].next;
  fi;
 od;
 Append(TupleTable,[rec(tuple:=T,next:=FingerprintTable[f].firstTuple)]);
 a:=Length(TupleTable);
 FingerprintTable[f].firstTuple:=a;
 return a;
end;


# End

